import 'dart:async';

import 'package:_private_chat/firestore_resouce/instances.dart';
import 'package:_private_core/_private_core.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_chat/constant.dart';

part 'connection_status_event.dart';
part 'connection_status_state.dart';

ConnectionStatusBloc get chatConnectionStatusBloc =>
    appFindInstance<ConnectionStatusBloc>();

class ConnectionStatusBloc
    extends Bloc<ConnectionStatusEvent, ConnectionStatusState> {
  ConnectionStatusBloc()
      : super(ConnectionStatusState(
            connectionStatusByKeys: {}, users: [], blocked: [], muted: [])) {
    on<ConnectionStatusInitEvent>(_onInit);
    on<ConnectionStatusUpdateEvent>(_onUpdate);
    on<ConnectionStatusSetStatusEvent>(_onSetStatus);
    on<ConnectionStatusBlockToEvent>(_onBlockTo);
    on<ConnectionStatusMuteToEvent>(_onMuteTo);
    on<ConnectionStatusDisposeEvent>(_onDispose);
  }

  StreamSubscription? _subscription;

  static setStatusReceived(firebaseId) async {
    if (!isLoggedFirebaseAuth()) return;
    colCONNECTIONSTATUS
        .doc(firebaseId)
        .update({kDbPERSONALRECEIVED: DateTime.now().millisecondsSinceEpoch});
  }

  _onDispose(ConnectionStatusDisposeEvent event, emit) async {
    _subscription?.cancel();
  }

  _onMuteTo(ConnectionStatusMuteToEvent event, emit) async {
    if (!isLoggedFirebaseAuth()) return;
    Map<String, dynamic> data = {};
    if (event.status) {
      data.addAll({
        kDbMUTED: List.from(state.muted + [event.id])
      });
    } else {
      data.addAll(
          {kDbMUTED: List.from(state.muted.where((e) => e != event.id))});
    }

    colCONNECTIONSTATUS
        .doc(loggedFirebaseId)
        .set(data, SetOptions(merge: true));
  }

  _onBlockTo(ConnectionStatusBlockToEvent event, emit) async {
    if (!isLoggedFirebaseAuth()) return;
    Map<String, dynamic> data = {};
    if (event.status) {
      data.addAll({
        kDbBLOCKED: List.from(state.blocked + [event.id])
      });
    } else {
      data.addAll(
          {kDbBLOCKED: List.from(state.blocked.where((e) => e != event.id))});
    }

    colCONNECTIONSTATUS
        .doc(loggedFirebaseId)
        .set(data, SetOptions(merge: true));
  }

  _onSetStatus(ConnectionStatusSetStatusEvent event, emit) async {
    if (!isLoggedFirebaseAuth()) return;
    Map<String, dynamic> data = {
      kDbFIREBASEID: loggedFirebaseId,
      kDbPERSONALRECEIVED: DateTime.now().millisecondsSinceEpoch
    };
    if (event.isOnline != null) {
      if (event.isOnline == true) {
        data[kDbPERSONAL] = true;
      } else {
        data[kDbPERSONAL] = DateTime.now().millisecondsSinceEpoch;
      }
      data[kDbCONVERSATION] = ConversationStatus.none.name;
      data[kDbCONVERSATIONID] = '';
    } else if (event.conversationStatus != null) {
      data[kDbPERSONAL] = true;
      data[kDbCONVERSATION] = event.conversationStatus!.name;
      if (event.conversationStatus == ConversationStatus.none) {
        data[kDbCONVERSATIONID] = '';
      } else {
        data[kDbCONVERSATIONID] = event.conversationId;
      }
    }

    colCONNECTIONSTATUS
        .doc(loggedFirebaseId)
        .set(data, SetOptions(merge: true));
  }

  _onUpdate(ConnectionStatusUpdateEvent event, emit) async {
    emit(state.update(
      connectionStatusByKeys: event.connectionStatusByKeys,
      muted: event.muted,
      blocked: event.blocked,
    ));
  }

  _onInit(ConnectionStatusInitEvent event, emit) async {
    _subscription?.cancel();
    _subscription = null;
    if (state.users != event.users && event.users.isNotEmpty) {
      emit(state.update(users: event.users));
      var query = await colCONNECTIONSTATUS
          .where(kDbFIREBASEID, whereIn: event.users)
          .get();
      _dataToState(query);

      _subscription?.cancel();
      _subscription = null;
      _subscription = colCONNECTIONSTATUS
          .where(kDbFIREBASEID, whereIn: event.users)
          .snapshots()
          .listen((query) {
        _dataToState(query);
      });
    }
  }

  _dataToState(QuerySnapshot query) {
    Map connectionStatusByKeys = {};
    for (var d in query.docs) {
      Map data = d.data() as Map;
      if (data[kDbFIREBASEID] == loggedFirebaseId) {
        add(ConnectionStatusUpdateEvent(
            muted: data[kDbMUTED], blocked: data[kDbBLOCKED]));
      } else if (state.users.contains(data[kDbFIREBASEID])) {
        if (data[kDbFIREBASEID] != data) {
          connectionStatusByKeys[data[kDbFIREBASEID]] = data;
          add(ConnectionStatusUpdateEvent(
              connectionStatusByKeys: connectionStatusByKeys));
        }
      }
    }
  }
}
