import 'dart:io';

import 'package:_private_core/_private_core.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';

class AppUploader {
  static Future<String> uploadFile({
    required Uint8List data,
    required String path,
    required String userId,
    bool isNeedCompress = false,
    SettableMetadata? meta,
    Function(TaskSnapshot)? onUploading,
  }) async {
    appDebugPrint("AppUploader started");
    // File name
    String assetName =
        '${userId}_${DateTime.now().millisecondsSinceEpoch.toString()}';

    if (isNeedCompress && (kIsWeb || Platform.isAndroid || Platform.isIOS || Platform.isMacOS)) {
      appDebugPrint("AppUploader Compressing");
      data = await FlutterImageCompress.compressWithList(data,
          minWidth: 600, minHeight: 600);
      appDebugPrint("AppUploader Compressed");
    }

    appDebugPrint("AppUploader Upload file");
    // Upload file
    final UploadTask uploadTask = FirebaseStorage.instance
        .ref()
        .child('$path/$userId/$assetName')
        .putData(data, meta);
    if (onUploading != null) {
      uploadTask.snapshotEvents.listen(onUploading);
    }
    final TaskSnapshot snapshot = await uploadTask;
    String url = await snapshot.ref.getDownloadURL();
    appDebugPrint("AppUploader Upload file done url=$url");
    return url;
  }
}
