import 'dart:io';
import 'dart:math';

import 'package:_private_chat/_private_chat.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/auth/repo.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import '../chat_detail/chat_helper.dart';
import '../chat_detail/message.dart';
import '../constant.dart';
import '../firestore_resouce/instances.dart';

bool get isBot => chatDetailBloc.params.isBot;

bool get isMobile =>
    !kIsWeb && !Platform.isMacOS && !Platform.isWindows && !Platform.isLinux;

bool isLoggedFirebaseAuth() {
  bool logged = FirebaseAuth.instance.currentUser != null;
  if (!logged) {
    appDebugPrint('[isLoggedFirebaseAuth] false');
  }
  return logged;
}

CommonUserModel kDefaultUserInfo(firebaseId) => CommonUserModel.fromJson({
      "username": "anonymous",
      "displayName": "Anonymous",
      "firebaseId": firebaseId,
      "isPro": 0,
      "thumbnail": "/thumbnails/default/user.jpg",
      "uniqueId": "Uanonymous",
      "isAnonymously": 1,
    });

int getTimestamp(Map data) => data[kDbTIMESTAMP];

String getContent(String key, Map data) =>
    ChatHelper.decrypt(key, data[kDbCONTENT]);

bool getIsMe(Map data) => data[kDbFROM] == loggedFirebaseId;

bool get isLoggedAnonymous => appPrefs.loginUser?.id == null;

String get loggedUsername => appPrefs.loginUser?.username ?? '';

String get loggedFirebaseId => appPrefs.loginUser?.firebaseId ?? '';

Future<CommonUserModel> getInfoByFirebaseId(String? firebaseId, {required isAnonymously}) async {
  if (!isAnonymously && firebaseId != null && firebaseId != "") {
    return (await AuthXRepo()
            .getUserProInfoByFirebaseId({"firebaseId": firebaseId})) ??
        kDefaultUserInfo(firebaseId);
  }
  return kDefaultUserInfo(firebaseId);
}

Future<Map> getStatisticFirstMessage(int platformId) async {
  QuerySnapshot q = await colCONVERSATIONS
      .where(kDbMEMBERSFIREBASEID, arrayContains: loggedFirebaseId)
      .orderBy(kDbLASTMESSAGETIME, descending: true)
      .get();
  var totalfirstMessage = 0;
  var totalfirstMessageAnswer = 0;
  var totalfirstMessageAnswerTime = 0;
  for (var d in q.docs) {
    Map docData = d.data() as Map;
    if (docData['firstMessage'] != null &&
        docData['firstMessage'][kDbPLATFORMID] == platformId &&
        docData['firstMessage'][kDbFROM] != loggedFirebaseId) {
      totalfirstMessage++;

      if (docData['firstMessageAnswer'] != null &&
          docData['firstMessageAnswer'][kDbFROM] == loggedFirebaseId) {
        if ((docData['firstMessageAnswer'][kDbTIMESTAMP] as int) >
            (docData['firstMessage'][kDbTIMESTAMP] as int)) {
          totalfirstMessageAnswerTime +=
              (docData['firstMessageAnswer'][kDbTIMESTAMP] as int) -
                  (docData['firstMessage'][kDbTIMESTAMP] as int);
        }
        totalfirstMessageAnswer++;
      }
    }
  }
  var result = {
    //in miliseconds
    "averageTimeResponse": totalfirstMessageAnswer == 0
        ? 0
        : totalfirstMessageAnswerTime / totalfirstMessageAnswer,
    "totalfirstMessage": totalfirstMessage,
    "pourcentageMessageResponse": totalfirstMessage == 0
        ? 0
        : ((totalfirstMessageAnswer / totalfirstMessage) * 100)
  };
  return result;
}

Future<Map?> getNextMessage(
    MessageType messageType, conversationId, timestamp) async {
  QuerySnapshot q = await colCONVERSATIONS
      .doc(conversationId)
      .collection(kCollectionMESSAGES)
      .where(kDbMESSAGETYPE, isEqualTo: messageType.index)
      .where(kDbTIMESTAMP, isGreaterThan: timestamp)
      .limit(1)
      .get();
  if (q.size != 0) {
    return q.docs.single.data() as Map;
  }
  return null;
}

String generateRandomString(int len) {
  var r = Random();
  const chars =
      'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890';
  return List.generate(len, (index) => chars[r.nextInt(chars.length)]).join();
}
